@@ -4,12 +4,12 @@
 # exports.py, provides classes for a file export dialog and export functions
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2019, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import os.path
@@ -120,7 +120,7 @@ class ExportControl:
         filters = ';;'.join((globalref.fileFilters[defaultExt],
                              globalref.fileFilters['all']))
         defaultExt = defaultExt[:4]
-        if self.defaultPathObj.name:
+        if self.defaultPathObj.is_file():
             self.defaultPathObj = self.defaultPathObj.with_suffix('.' +
                                                                   defaultExt)
         filePath, selectFilter = QFileDialog.getSaveFileName(QApplication.
@@ -782,8 +782,9 @@ class ExportControl:
             for data in nodeData:
                 data['children'] = []
             fileData['nodes'] = nodeData
+        indent = 3 if globalref.genOptions['PrettyPrint'] else 0
         with pathObj.open('w', encoding='utf-8', newline='\n') as f:
-            json.dump(fileData, f, indent=0, sort_keys=True)
+            json.dump(fileData, f, indent=indent, sort_keys=True)
         return True
 
     def exportXmlGeneric(self, pathObj=None):
@@ -1187,7 +1188,7 @@ def _writeHtmlTable(node, parent, pathDict, level=1):
                   node.childList[0].formatRef.fieldNames()])
     lines.append('</tr><tr>')
     for child in node.childList:
-        cellList = [field.outputText(child, False, True) for field in
+        cellList = [field.outputText(child, False, False, True) for field in
                     child.formatRef.fields()]
         for i in range(len(cellList)):
             startPos = 0
@@ -1336,7 +1337,7 @@ def _setOldUniqueId(idDict, node):
     """
     nodeFormat = node.formatRef
     idField = next(iter(nodeFormat.fieldDict.values()))
-    uId = idField.outputText(node, True, nodeFormat.formatHtml)
+    uId = idField.outputText(node, True, True, nodeFormat.formatHtml)
     uId = uId.strip().split('\n', 1)[0]
     maxLength = 50
     if len(uId) > maxLength:
