@@ -4,21 +4,27 @@
 # matheval.py, provides a safe eval of mathematical expressions
 #
 # TreeLine, an information storage program
-# Copyright (C) 2017, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import re
 import ast
 import enum
+import datetime
 import builtins
+import fieldformat
 import gennumber
 from math import *
 
+_nowDateString = 'Now_Date'
+_nowTimeString = 'Now_Time'
+_nowDateTimeString = 'Now_Date_Time'
+
 
 def sum(*args):
     """Override the builtin sum function to handle multiple arguments.
@@ -205,21 +211,34 @@ class MathEquation:
         except ZeroDivisionError:
             pass
 
-    def equationValue(self, eqnNode, zeroValue=0):
+    def equationValue(self, eqnNode, resultType, zeroValue=0, noMarkup=True):
         """Return a value for the equation in the given node.
 
         Return None if references are invalid.
         Raise a ValueError for illegal math operations.
         Arguments:
             eqnNode -- the node containing the equation to evaluate
+            resultType -- the result type from fieldformat
             zeroValue -- the value to use for blanks
+            noMarkup -- if true, remove html markup
         """
         zeroBlanks = eqnNode.treeStructureRef().mathZeroBlanks
-        inputs = [ref.referenceValue(eqnNode, zeroBlanks) for ref in
-                  self.fieldRefs]
-        if not zeroBlanks and None in inputs:
-            return None
-        inputs = [repr(value) for value in inputs]
+        checker = SafeEvalChecker()
+        inputs = []
+        for ref in self.fieldRefs:
+            inp = ref.referenceValue(eqnNode, zeroBlanks, zeroValue, noMarkup)
+            if inp == None and not zeroBlanks:
+                return None
+            if (resultType == fieldformat.MathResult.number and
+                hasattr(inp, 'format')):
+                try:
+                    checker.check(inp)
+                    inp = eval(inp)
+                except Exception:
+                    inp = repr(inp)
+            else:
+                inp = repr(inp)
+            inputs.append(inp)
         eqn = self.formattedEqnText.format(*inputs)
         try:
             return eval(eqn)
@@ -254,7 +273,7 @@ class MathEquation:
 
 
 # recursive equation ref eval directions
-EvalDir = enum.IntEnum('EvalDir', 'optional upward downward')
+EvalDir = enum.IntEnum('EvalDir', 'downward upward optional')
 
 
 class EquationFieldRef:
@@ -275,7 +294,8 @@ class EquationFieldRef:
         self.eqnNodeTypeName = ''
         self.eqnFieldName = ''
 
-    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
+    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0,
+                       noMarkup=True):
         """Return the value of the field referenced in a given node.
 
         Return None if blank or doesn't exist and not zeroBlanks,
@@ -284,11 +304,24 @@ class EquationFieldRef:
             eqnNode -- the node containing the equation to evaluate
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
+            noMarkup -- if true, remove html markup
         """
         try:
             return (eqnNode.formatRef.fieldDict[self.fieldName].
-                    mathValue(eqnNode, zeroBlanks))
+                    mathValue(eqnNode, zeroBlanks, noMarkup))
         except KeyError:
+            if self.fieldName == _nowDateString:
+                return (datetime.date.today() -
+                        fieldformat.DateField.refDate).days
+            elif self.fieldName == _nowTimeString:
+                now = datetime.datetime.combine(fieldformat.DateField.refDate,
+                                                datetime.datetime.now().time())
+                ref = datetime.datetime.combine(fieldformat.DateField.refDate,
+                                                fieldformat.TimeField.refTime)
+                return (now - ref).seconds
+            elif self.fieldName == _nowDateTimeString:
+                return (datetime.datetime.now() -
+                        fieldformat.DateTimeField.refDateTime).total_seconds()
             return zeroValue if zeroBlanks else None
 
     def dependentEqnNodes(self, refNode):
@@ -309,7 +342,8 @@ class EquationParentRef(EquationFieldRef):
     testValue = 1
     evalDirection = EvalDir.downward
 
-    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
+    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0,
+                       noMarkup=True):
         """Return the parent field value referenced from a given node.
 
         Return None if blank or doesn't exist and not zeroBlanks,
@@ -318,13 +352,14 @@ class EquationParentRef(EquationFieldRef):
             eqnNode -- the node containing the equation to evaluate
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
+            noMarkup -- if true, remove html markup
         """
         node = eqnNode.spotByNumber(0).parentSpot.nodeRef
         if not node.formatRef:
             return zeroValue if zeroBlanks else None
         try:
             return (node.formatRef.fieldDict[self.fieldName].
-                    mathValue(node, zeroBlanks))
+                    mathValue(node, zeroBlanks, noMarkup))
         except KeyError:
             return zeroValue if zeroBlanks else None
 
@@ -345,7 +380,8 @@ class EquationRootRef(EquationFieldRef):
     testValue = 1
     evalDirection = EvalDir.downward
 
-    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
+    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0,
+                       noMarkup=True):
         """Return the root field value referenced from a given node.
 
         Return None if blank or doesn't exist and not zeroBlanks,
@@ -354,11 +390,12 @@ class EquationRootRef(EquationFieldRef):
             eqnNode -- the node containing the equation to evaluate
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
+            noMarkup -- if true, remove html markup
         """
         node = eqnNode.spotByNumber(0).spotChain()[0].nodeRef
         try:
             return (node.formatRef.fieldDict[self.fieldName].
-                    mathValue(node, zeroBlanks))
+                    mathValue(node, zeroBlanks, noMarkup))
         except KeyError:
             return zeroValue if zeroBlanks else None
 
@@ -373,7 +410,7 @@ class EquationRootRef(EquationFieldRef):
             return []
         refs = [node for node in refNode.descendantGen() if
                 node.formatRef.name == self.eqnNodeTypeName]
-        if refs[0] is refNode:
+        if refs and refs[0] is refNode:
             refs = refs[1:]
         return refs
 
@@ -385,7 +422,8 @@ class EquationChildRef(EquationFieldRef):
     testValue = [1]
     evalDirection = EvalDir.upward
 
-    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
+    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0,
+                       noMarkup=True):
         """Return a list with child field values referenced from a given node.
 
         Return None if there are blanks and zeroBlanks is false,
@@ -399,7 +437,7 @@ class EquationChildRef(EquationFieldRef):
         for node in eqnNode.childList:
             try:
                 num = (node.formatRef.fieldDict[self.fieldName].
-                       mathValue(node, zeroBlanks))
+                       mathValue(node, zeroBlanks, noMarkup))
                 if num == None:
                     return None
                 result.append(num)
@@ -429,13 +467,15 @@ class EquationChildCountRef(EquationFieldRef):
     testValue = 1
     evalDirection = EvalDir.optional
 
-    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0):
+    def referenceValue(self, eqnNode, zeroBlanks=True, zeroValue=0,
+                       noMarkup=True):
         """Return the child count referenced from the given node.
 
         Arguments:
             eqnNode -- the node containing the equation to evaluate
             zeroBlanks -- replace blank fields with zeroValue if True
             zeroValue -- the value to use for blanks
+            noMarkup -- if true, remove html markup
         """
         return len(eqnNode.childList)
 
@@ -484,6 +524,7 @@ class RecursiveEqnRef:
         self.evalSequence = 1
         for fieldRef in self.eqnField.equation.fieldRefs:
             if (fieldRef.fieldName in visitedFields and
+                fieldRef.tagPrefix != '#' and
                 (self.eqnField.name != fieldRef.fieldName or
                  fieldRef.evalDirection == EvalDir.optional)):
                 raise CircularMathError()
@@ -521,12 +562,13 @@ allowedFunctions = set(['abs', 'float', 'int', 'len', 'max', 'min', 'pow',
                         'startswith', 'endswith', 'contains',
                         'join', 'upper', 'lower', 'replace'])
 
-allowedNodeTypes = set(['Module', 'Expr', 'Name', 'Load', 'IfExp', 'Compare',
-                        'Num', 'Str', 'Tuple', 'List', 'BinOp', 'UnaryOp',
-                        'Add', 'Sub', 'Mult', 'Div', 'Mod', 'Pow', 'FloorDiv',
-                        'Invert', 'Not', 'UAdd', 'USub',
-                        'Eq', 'NotEq', 'Lt', 'LtE', 'Gt', 'GtE', 'Is', 'IsNot',
-                        'In', 'NotIn', 'BoolOp', 'And', 'Or'])
+allowedNodeTypes = set(['Module', 'Expr', 'Name', 'NameConstant', 'Constant',
+                        'Load', 'IfExp', 'Compare', 'Num', 'Str', 'Tuple',
+                        'List', 'BinOp', 'UnaryOp', 'Add', 'Sub', 'Mult',
+                        'Div', 'Mod', 'Pow', 'FloorDiv', 'Invert', 'Not',
+                        'UAdd', 'USub', 'Eq', 'NotEq', 'Lt', 'LtE', 'Gt',
+                        'GtE', 'Is', 'IsNot', 'In', 'NotIn', 'BoolOp',
+                        'And', 'Or'])
 
 
 class SafeEvalChecker(ast.NodeVisitor):
