@@ -4,12 +4,12 @@
 # miscdialogs.py, provides classes for various control dialogs
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import enum
@@ -794,7 +794,8 @@ class FindReplaceDialog(QDialog):
     def updateAvail(self):
         """Set find & replace buttons available if search text & matches exist.
         """
-        hasEntry = len(self.textEntry.text().strip()) > 0
+        hasEntry = (len(self.textEntry.text().strip()) > 0 or
+                    self.howButtons.checkedId() == FindReplaceType.anyMatch)
         self.previousButton.setEnabled(hasEntry)
         self.nextButton.setEnabled(hasEntry)
         match = bool(self.matchedSpot and self.matchedSpot is
@@ -881,7 +882,7 @@ class FindReplaceDialog(QDialog):
             searchText, regExpObj, typeName, fieldName = self.findParameters()
         except re.error:
             QMessageBox.warning(self, 'TreeLine',
-                                      _('Error - invalid regular expression'))
+                                _('Error - invalid regular expression'))
             self.updateAvail()
             return
         control = globalref.mainControl.activeControl
@@ -1216,8 +1217,6 @@ class NumberingDialog(QDialog):
                 node.updateNumbering(fieldDict, startNum, levelLimit,
                                      completedClones, includeRoot,
                                      reserveNums, restartSetting)
-                if not restartSetting:
-                    startNum[0] += 1
             control.updateAll()
         QApplication.restoreOverrideCursor()
 
@@ -1907,11 +1906,13 @@ class CustomFontDialog(QDialog):
                 defaultFont = QFont(globalref.mainControl.systemFont)
             else:
                 defaultFont = appFontWidget.readFont()
-            currentWidget.printData.defaultFont = defaultFont
-            if currentWidget.defaultCheck.isChecked():
-                currentWidget.printData.mainFont = QFont(defaultFont)
-                currentWidget.currentFont = currentWidget.printData.mainFont
-                currentWidget.setFont(defaultFont)
+            if defaultFont:
+                currentWidget.printData.defaultFont = defaultFont
+                if currentWidget.defaultCheck.isChecked():
+                    currentWidget.printData.mainFont = QFont(defaultFont)
+                    currentWidget.currentFont = (currentWidget.printData.
+                                                 mainFont)
+                    currentWidget.setFont(defaultFont)
 
     def applyChanges(self):
         """Apply any changes from the dialog.
