@@ -4,19 +4,20 @@
 # printdata.py, provides a class for printing
 #
 # TreeLine, an information storage program
-# Copyright (C) 2017, Douglas W. Bell
+# Copyright (C) 2019, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import os.path
 import enum
 from PyQt5.QtCore import QMarginsF, QSizeF, Qt
 from PyQt5.QtGui import (QAbstractTextDocumentLayout, QFontMetrics,
-                         QPageLayout, QPageSize, QPainter, QTextDocument)
+                         QPageLayout, QPageSize, QPainter, QPalette,
+                         QTextDocument)
 from PyQt5.QtWidgets import QApplication, QDialog, QFileDialog, QMessageBox
 from PyQt5.QtPrintSupport import QPrintDialog, QPrinter
 import treeoutput
@@ -220,6 +221,7 @@ class PrintData:
         pageNum = 1
         columnNum = 0
         pagePos = 0
+        itemSplit = False
         self.checkPageLayout()
         heightAvail = (self.pageLayout.paintRect().height() *
                        self.printer.logicalDpiY())
@@ -255,7 +257,7 @@ class PrintData:
             if item:
                 item.setDocHeight(self.printer, widthRemain, self.mainFont,
                                   True)
-                if item.height > heightAvail:
+                if item.height > heightAvail and not itemSplit:
                     item, newItem = item.splitDocHeight(heightAvail - pagePos,
                                                         heightAvail,
                                                         self.printer,
@@ -263,6 +265,7 @@ class PrintData:
                                                         self.mainFont)
                     if newItem:
                         self.outputGroup.insert(0, newItem)
+                        itemSplit = True
             if item and (pagePos + item.height <= heightAvail or pagePos == 0):
                 item.pageNum = pageNum
                 item.columnNum = columnNum
@@ -276,6 +279,7 @@ class PrintData:
                     pageNum += 1
                     columnNum = 0
                 pagePos = 0
+                itemSplit = False
                 if item:
                     self.outputGroup.insert(0, item)
                     if self.widowControl and not item.siblingPrefix:
@@ -291,7 +295,8 @@ class PrintData:
                             moveItems.insert(0, newGroup.pop())
                             moveHeight += moveItems[0].height
                             level -= 1
-                        if moveItems and moveHeight < (heightAvail // 5):
+                        if (moveItems and newGroup and
+                            moveHeight < (heightAvail // 5)):
                             self.outputGroup[0:0] = moveItems
                         else:
                             newGroup.extend(moveItems)
@@ -352,7 +357,10 @@ class PrintData:
         """Paint data to be printed to the printer.
         """
         pageNum = 1
-        maxPageNum = self.outputGroup[-1].pageNum
+        try:
+            maxPageNum = self.outputGroup[-1].pageNum
+        except IndexError:   # printing empty branch
+            maxPageNum = 1
         if self.printer.printRange() != QPrinter.AllPages:
             pageNum = self.printer.fromPage()
             maxPageNum = self.printer.toPage()
@@ -376,7 +384,14 @@ class PrintData:
             pageNum -- the page number to be printed
             painter -- the painter for this print job
         """
-        totalNumPages = self.outputGroup[-1].pageNum
+        paintContext = QAbstractTextDocumentLayout.PaintContext()
+        # set context text color to black to wrok with dark app themes
+        paintContext.palette = QPalette()
+        paintContext.palette.setColor(QPalette.Text, Qt.black)
+        try:
+            totalNumPages = self.outputGroup[-1].pageNum
+        except IndexError:   # printing empty branch
+            totalNumPages = 1
         headerDoc = self.headerFooterDoc(True, pageNum, totalNumPages)
         if headerDoc:
             layout = headerDoc.documentLayout()
@@ -388,8 +403,7 @@ class PrintData:
             headerDelta = ((self.headerMargin - topMargin) *
                            self.printer.logicalDpiX())
             painter.translate(0, int(headerDelta))
-            layout.draw(painter,
-                        QAbstractTextDocumentLayout.PaintContext())
+            layout.draw(painter, paintContext)
             painter.restore()
         painter.save()
         columnSpacing = int(self.columnSpacing * self.printer.logicalDpiX())
@@ -400,7 +414,7 @@ class PrintData:
         for columnNum in range(self.numColumns):
             if columnNum > 0:
                 painter.translate(columnDelta, 0)
-            self.paintColumn(pageNum, columnNum, painter)
+            self.paintColumn(pageNum, columnNum, painter, paintContext)
         painter.restore()
         footerDoc = self.headerFooterDoc(False, pageNum, totalNumPages)
         if footerDoc:
@@ -415,11 +429,10 @@ class PrintData:
             painter.translate(0, self.pageLayout.paintRect().height() *
                               self.printer.logicalDpiX() +
                               int(footerDelta) - self.lineSpacing)
-            layout.draw(painter,
-                        QAbstractTextDocumentLayout.PaintContext())
+            layout.draw(painter, paintContext)
             painter.restore()
 
-    def paintColumn(self, pageNum, columnNum, painter):
+    def paintColumn(self, pageNum, columnNum, painter, paintContext):
         """Paint data for the given column to the printer.
 
         Arguments:
@@ -433,8 +446,7 @@ class PrintData:
             layout = item.doc.documentLayout()
             painter.save()
             painter.translate(item.level * self.indentSize, item.pagePos)
-            layout.draw(painter,
-                        QAbstractTextDocumentLayout.PaintContext())
+            layout.draw(painter, paintContext)
             painter.restore()
         if self.drawLines:
             self.addPrintLines(pageNum, columnNum, columnItems, painter)
@@ -457,8 +469,9 @@ class PrintData:
             if item.level > 0:
                 indent = item.level * self.indentSize
                 vertPos = item.pagePos + vertOffset
-                painter.drawLine(indent - horizOffset, vertPos,
-                                 indent - self.lineSpacing // 4, vertPos)
+                painter.drawLine(int(indent - horizOffset), int(vertPos),
+                                 int(indent - self.lineSpacing // 4),
+                                 int(vertPos))
                 parent = item.parentItem
                 while parent:
                     if parent in parentsDrawn:
@@ -476,8 +489,8 @@ class PrintData:
                          parent.lastChildItem.columnNum >= columnNum)):
                         horizPos = ((parent.level + 1) * self.indentSize -
                                     horizOffset)
-                        painter.drawLine(horizPos, lineStart,
-                                         horizPos, lineEnd)
+                        painter.drawLine(int(horizPos), int(lineStart),
+                                         int(horizPos), int(lineEnd))
                     parentsDrawn.add(parent)
                     parent = parent.parentItem
 
