@@ -4,16 +4,16 @@
 # treeline.py, the main program file
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2019, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 __progname__ = 'TreeLine'
-__version__ = '3.0.1'
+__version__ = '3.1.4'
 __author__ = 'Doug Bell'
 
 docPath = None         # modified by install script if required
@@ -41,9 +41,10 @@ def loadTranslator(fileName, app):
         app -- the main QApplication
     """
     translator = QTranslator(app)
-    modPath = pathlib.Path(sys.path[0]).resolve()
+    # use abspath() - pathlib's resolve() can be buggy with network drives
+    modPath = pathlib.Path(os.path.abspath(sys.path[0]))
     if modPath.is_file():
-        modPath = modPath.parent
+        modPath = modPath.parent  # for frozen binary
     path = modPath / translationPath
     result = translator.load(fileName, str(path))
     if not result:
@@ -149,7 +150,7 @@ if __name__ == '__main__':
     parser.add_argument('fileList', nargs='*', metavar='filename',
                         help='input filename(s) to load')
     args = parser.parse_args()
-    # use abspath() - pathlib's resolve() replaces drive letters with map names
+    # use abspath() - pathlib's resolve() can be buggy with network drives
     pathObjects = [pathlib.Path(os.path.abspath(path)) for path in
                    args.fileList]
 
