@@ -4,12 +4,12 @@
 # treelocalcontrol.py, provides a class for the main tree commands
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import pathlib
@@ -114,13 +114,10 @@ class TreeLocalControl(QObject):
             else:
                 oldControl.controlClosed.emit(oldControl)
             window.resetTreeModel(self.model)
-            # set first spot current to match cases with a new window
-            window.treeView.selectionModel().setCurrentSpot(self.structure.
-                                                            rootSpots()[0])
             self.setWindowSignals(window, True)
             window.updateActions(self.allActions)
             self.windowList.append(window)
-            window.setCaption(self.filePathObj)
+            self.updateWindowCaptions()
             self.activeWindow = window
         if fileObj and self.structure.childRefErrorNodes:
             msg = _('Warning - file corruption!\n'
@@ -196,14 +193,17 @@ class TreeLocalControl(QObject):
             self.setModified()
         QApplication.restoreOverrideCursor()
 
-    def updateRightViews(self, setModified=False):
+    def updateRightViews(self, setModified=False, otherTrees=False):
         """Update the right-hand views in all windows.
 
         Arguments:
             setModified -- if True, set the modified flag for this file
+            otherTrees -- if True, also update trees in non-active windows
         """
         for window in self.windowList:
             window.updateRightViews()
+            if otherTrees and window != self.activeWindow:
+                window.updateTree()
         if setModified:
             self.setModified()
 
@@ -300,13 +300,15 @@ class TreeLocalControl(QObject):
             self.allActions['NodeMoveDown'].setEnabled(False)
             self.allActions['NodeMoveFirst'].setEnabled(False)
             self.allActions['NodeMoveLast'].setEnabled(False)
+        else:
+            self.allActions['NodeAddChild'].setEnabled(True)
         self.activeWindow.updateCommandsAvail()
 
     def updateWindowCaptions(self):
         """Update the caption for all windows.
         """
         for window in self.windowList:
-            window.setCaption(self.filePathObj)
+            window.setCaption(self.filePathObj, self.modified)
 
     def setModified(self, modified=True):
         """Set the modified flag on this file and update commands available.
@@ -317,6 +319,7 @@ class TreeLocalControl(QObject):
         if modified != self.modified:
             self.modified = modified
             self.allActions['FileSave'].setEnabled(modified)
+            self.updateWindowCaptions()
             self.resetAutoSave()
 
     def expandRootNodes(self, maxNum=5):
@@ -330,6 +333,14 @@ class TreeLocalControl(QObject):
             for spot in self.structure.rootSpots():
                 treeView.expandSpot(spot)
 
+    def selectRootSpot(self):
+        """Select the first root spot in the tree.
+
+        Does not signal an update.
+        """
+        self.currentSelectionModel().selectSpots([self.structure.
+                                                  rootSpots()[0]], False)
+
     def currentSelectionModel(self):
         """Return the current tree's selection model.
         """
@@ -629,6 +640,11 @@ class TreeLocalControl(QObject):
         dataCopyTypeAct.triggered.connect(self.dataCopyType)
         localActions['DataCopyType'] = dataCopyTypeAct
 
+        dataRegenRefsAct = QAction(_('&Regenerate References'), self,
+            statusTip=_('Force update of all conditional types & math fields'))
+        dataRegenRefsAct.triggered.connect(self.dataRegenRefs)
+        localActions['DataRegenRefs'] = dataRegenRefsAct
+
         dataCloneMatchesAct = QAction(_('Clone All &Matched Nodes'), self,
                          statusTip=_('Convert all matching nodes into clones'))
         dataCloneMatchesAct.triggered.connect(self.dataCloneMatches)
@@ -709,6 +725,11 @@ class TreeLocalControl(QObject):
         formatIntLinkAct.setEnabled(False)
         localActions['FormatIntLink'] = formatIntLinkAct
 
+        formatInsDateAct = QAction(_('Insert &Date'), self,
+                                   statusTip=_('Insert current date as text'))
+        formatInsDateAct.setEnabled(False)
+        localActions['FormatInsertDate'] = formatInsDateAct
+
         formatClearFormatAct =  QAction(_('Clear For&matting'), self,
                       statusTip=_('Clear current or selected text formatting'))
         formatClearFormatAct.setEnabled(False)
@@ -754,10 +775,11 @@ class TreeLocalControl(QObject):
         if self.spellCheckLang:
             fileData['properties']['spellchk'] = self.spellCheckLang
         if not self.compressed and not self.encrypted:
+            indent = 3 if globalref.genOptions['PrettyPrint'] else 0
             try:
                 with savePathObj.open('w', encoding='utf-8',
                                       newline='\n') as f:
-                    json.dump(fileData, f, indent=0, sort_keys=True)
+                    json.dump(fileData, f, indent=indent, sort_keys=True)
             except IOError:
                 QApplication.restoreOverrideCursor()
                 QMessageBox.warning(self.activeWindow, 'TreeLine',
@@ -815,7 +837,8 @@ class TreeLocalControl(QObject):
                              globalref.fileFilters['trlnenc']))
         initFilter = globalref.fileFilters['trlnsave']
         defaultPathObj = globalref.mainControl.defaultPathObj()
-        defaultPathObj = defaultPathObj.with_suffix('.trln')
+        if defaultPathObj.is_file():
+            defaultPathObj = defaultPathObj.with_suffix('.trln')
         newPath, selectFilter = (QFileDialog.
                                  getSaveFileName(self.activeWindow,
                                                  _('TreeLine - Save As'),
@@ -1265,6 +1288,11 @@ class TreeLocalControl(QObject):
         self.updateAll()
         globalref.mainControl.updateConfigDialog()
 
+    def dataRegenRefs(self):
+        """Force update of all conditional types & math fields.
+        """
+        self.updateAll(False)
+
     def dataCloneMatches(self):
         """Convert all matching nodes into clones.
         """
@@ -1532,8 +1560,8 @@ class TreeLocalControl(QObject):
         Called from the find replace dialog.
         Returns True if found, otherwise False.
         Arguments:
-            searchText -- the text to find in a non-regexp search
-            regExpObj -- the regular expression to find if searchText is blank
+            searchText -- the text to find if no regexp is given
+            regExpObj -- the regular expression to find if given
             typeName -- if given, verify that this node matches this type
             fieldName -- if given, only find matches under this type name
             forward -- next if True, previous if False
@@ -1594,8 +1622,8 @@ class TreeLocalControl(QObject):
         Called from the find replace dialog.
         Returns True if replaced, otherwise False.
         Arguments:
-            searchText -- the text to find in a non-regexp search
-            regExpObj -- the regular expression to find if searchText is blank
+            searchText -- the text to find if no regexp is given
+            regExpObj -- the regular expression to find if given
             typeName -- if given, verify that this node matches this type
             fieldName -- if given, only find matches under this type name
             replaceText -- if not None, replace a match with this string
@@ -1631,8 +1659,8 @@ class TreeLocalControl(QObject):
         Called from the find replace dialog.
         Returns number of matches replaced.
         Arguments:
-            searchText -- the text to find in a non-regexp search
-            regExpObj -- the regular expression to find if searchText is blank
+            searchText -- the text to find if no regexp is given
+            regExpObj -- the regular expression to find if given
             typeName -- if given, verify that this node matches this type
             fieldName -- if given, only find matches under this type name
             replaceText -- if not None, replace a match with this string
@@ -1663,13 +1691,19 @@ class TreeLocalControl(QObject):
         """
         window = treewindow.TreeWindow(self.model, self.allActions)
         self.setWindowSignals(window)
+        window.winMinimized.connect(globalref.mainControl.trayMinimize)
         self.windowList.append(window)
-        window.setCaption(self.filePathObj)
+        self.updateWindowCaptions()
         oldControl = globalref.mainControl.activeControl
         if oldControl:
-            oldControl.activeWindow.saveWindowGeom()
+            try:
+                oldControl.activeWindow.saveWindowGeom()
+            except RuntimeError:
+                # possibly avoid rare error of deleted c++ TreeWindow
+                pass
         window.restoreWindowGeom(offset)
         self.activeWindow = window
         self.expandRootNodes()
+        self.selectRootSpot()
         window.show()
         window.updateRightViews()
