@@ -4,26 +4,27 @@
 # treemaincontrol.py, provides a class for global tree commands
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import sys
 import pathlib
+import os.path
 import ast
 import io
 import gzip
 import zlib
 import platform
 from PyQt5.QtCore import QIODevice, QObject, Qt, PYQT_VERSION_STR, qVersion
-from PyQt5.QtGui import QFont
+from PyQt5.QtGui import QColor, QFont, QPalette
 from PyQt5.QtNetwork import QLocalServer, QLocalSocket
 from PyQt5.QtWidgets import (QAction, QApplication, QDialog, QFileDialog,
-                             QMessageBox, qApp)
+                             QMessageBox, QStyleFactory, QSystemTrayIcon, qApp)
 import globalref
 import treelocalcontrol
 import options
@@ -35,6 +36,7 @@ import imports
 import configdialog
 import miscdialogs
 import conditional
+import colorset
 import helpview
 try:
     from __main__ import __version__, __author__
@@ -67,6 +69,8 @@ class TreeMainControl(QObject):
         super().__init__(parent)
         self.localControls = []
         self.activeControl = None
+        self.trayIcon = None
+        self.isTrayMinimized = False
         self.configDialog = None
         self.sortDialog = None
         self.numberingDialog = None
@@ -77,6 +81,7 @@ class TreeMainControl(QObject):
         self.filterConditionDialog = None
         self.basicHelpView = None
         self.passwords = {}
+        self.creatingLocalControlFlag = False
         globalref.mainControl = self
         self.allActions = {}
         try:
@@ -93,6 +98,8 @@ class TreeMainControl(QObject):
                     sys.exit(0)
             # start local server to listen for attempt to start new session
             self.serverSocket = QLocalServer()
+            # remove any old servers still around after a crash in linux
+            self.serverSocket.removeServer('treeline3-session')
             self.serverSocket.listen('treeline3-session')
             self.serverSocket.newConnection.connect(self.getSocket)
         except AttributeError:
@@ -116,9 +123,13 @@ class TreeMainControl(QObject):
             globalref.toolbarOptions.readFile()
             globalref.keyboardOptions.readFile()
         except IOError:
+            errorDir = options.Options.basePath
+            if not errorDir:
+                errorDir = _('missing directory')
             QMessageBox.warning(None, 'TreeLine',
                                 _('Error - could not write config file to {}').
-                                format(options.Options.basePath))
+                                format(errorDir))
+            options.Options.basePath = None
         iconPathList = self.findResourcePaths('icons', iconPath)
         globalref.toolIcons = icondict.IconDict([path / 'toolbar' for path
                                                  in iconPathList],
@@ -129,6 +140,10 @@ class TreeMainControl(QObject):
             QApplication.setWindowIcon(windowIcon)
         globalref.treeIcons = icondict.IconDict(iconPathList, ['', 'tree'])
         icon = globalref.treeIcons.getIcon('default')
+        qApp.setStyle(QStyleFactory.create('Fusion'))
+        self.colorSet = colorset.ColorSet()
+        if globalref.miscOptions['ColorTheme'] != 'system':
+            self.colorSet.setAppColors()
         self.recentFiles = recentfiles.RecentFileList()
         if globalref.genOptions['AutoFileOpen'] and not pathObjects:
             recentPath = self.recentFiles.firstPath()
@@ -137,6 +152,8 @@ class TreeMainControl(QObject):
         self.setupActions()
         self.systemFont = QApplication.font()
         self.updateAppFont()
+        if globalref.genOptions['MinToSysTray']:
+            self.createTrayIcon()
         qApp.focusChanged.connect(self.updateActionsAvail)
         if pathObjects:
             for pathObj in pathObjects:
@@ -156,10 +173,14 @@ class TreeMainControl(QObject):
                 paths = ast.literal_eval(data)
                 if paths:
                     for path in paths:
-                        self.openFile(pathlib.Path(path), True)
+                        pathObj = pathlib.Path(path)
+                        if pathObj != self.activeControl.filePathObj:
+                            self.openFile(pathObj, True)
+                        else:
+                            self.activeControl.activeWindow.activateAndRaise()
                 else:
                     self.activeControl.activeWindow.activateAndRaise()
-            except(SyntaxError, ValueError, TypeError):
+            except(SyntaxError, ValueError, TypeError, RuntimeError):
                 pass
 
     def findResourcePaths(self, resourceName, preferredPath=''):
@@ -170,14 +191,18 @@ class TreeMainControl(QObject):
             resourceName -- the typical name of the resource directory
             preferredPath -- add this as the second path if given
         """
-        modPath = pathlib.Path(sys.path[0]).resolve()
-        basePath = pathlib.Path(options.Options.basePath)
-        pathList = [basePath / resourceName, modPath / '..' / resourceName,
-                    modPath / resourceName]
+        # use abspath() - pathlib's resolve() can be buggy with network drives
+        modPath = pathlib.Path(os.path.abspath(sys.path[0]))
+        if modPath.is_file():
+            modPath = modPath.parent    # for frozen binary
+        pathList = [modPath / '..' / resourceName, modPath / resourceName]
+        if options.Options.basePath:
+            basePath = pathlib.Path(options.Options.basePath)
+            pathList.insert(0, basePath / resourceName)
         if preferredPath:
             pathList.insert(1, pathlib.Path(preferredPath))
-        return [path.resolve() for path in pathList if path.is_dir() and
-                list(path.iterdir())]
+        return [pathlib.Path(os.path.abspath(str(path))) for path in pathList
+                if path.is_dir() and list(path.iterdir())]
 
     def findResourceFile(self, fileName, resourceName, preferredPath=''):
         """Return a path object for a resource file.
@@ -251,6 +276,7 @@ class TreeMainControl(QObject):
             if not (globalref.genOptions['SaveTreeStates'] and
                     self.recentFiles.retrieveTreeState(self.activeControl)):
                 self.activeControl.expandRootNodes()
+                self.activeControl.selectRootSpot()
             QApplication.restoreOverrideCursor()
         except IOError:
             QApplication.restoreOverrideCursor()
@@ -278,6 +304,7 @@ class TreeMainControl(QObject):
                             self.recentFiles.retrieveTreeState(self.
                                                                activeControl)):
                         self.activeControl.expandRootNodes()
+                        self.activeControl.selectRootSpot()
                     self.activeControl.compressed = compressed
                     self.activeControl.encrypted = encrypted
                     QApplication.restoreOverrideCursor()
@@ -433,6 +460,7 @@ class TreeMainControl(QObject):
             treeStruct -- the imported structure to use
             forceNewWindow -- if True, use a new window regardless of option
         """
+        self.creatingLocalControlFlag = True
         localControl = treelocalcontrol.TreeLocalControl(self.allActions,
                                                          pathObj, treeStruct,
                                                          forceNewWindow)
@@ -440,6 +468,7 @@ class TreeMainControl(QObject):
         localControl.controlClosed.connect(self.removeLocalControlRef)
         self.localControls.append(localControl)
         self.updateLocalControlRef(localControl)
+        self.creatingLocalControlFlag = False
         localControl.updateRightViews()
         localControl.updateCommandsAvail()
 
@@ -463,10 +492,13 @@ class TreeMainControl(QObject):
         Arguments:
             localControl -- the local control that is closing
         """
-        self.localControls.remove(localControl)
+        try:
+            self.localControls.remove(localControl)
+        except ValueError:
+            return  # skip for unreporducible bug - odd race condition?
         if globalref.genOptions['SaveTreeStates']:
             self.recentFiles.saveTreeState(localControl)
-        if not self.localControls:
+        if not self.localControls and not self.creatingLocalControlFlag:
             if globalref.genOptions['SaveWindowGeom']:
                 localControl.windowList[0].saveWindowGeom()
             else:
@@ -474,8 +506,59 @@ class TreeMainControl(QObject):
             self.recentFiles.writeItems()
             localControl.windowList[0].saveToolbarPosition()
             globalref.histOptions.writeFile()
+            if self.trayIcon:
+                self.trayIcon.hide()
+            # stop listening for session connections
+            try:
+                self.serverSocket.close()
+                del self.serverSocket
+            except AttributeError:
+                pass
+        if self.localControls:
+            # make sure a window is active (may not be focused), to avoid
+            # bugs due to a deleted current window
+            newControl = self.localControls[0]
+            newControl.setActiveWin(newControl.windowList[0])
         localControl.deleteLater()
 
+    def createTrayIcon(self):
+        """Create a new system tray icon if not already created.
+        """
+        if QSystemTrayIcon.isSystemTrayAvailable:
+            if not self.trayIcon:
+                self.trayIcon = QSystemTrayIcon(qApp.windowIcon(), qApp)
+                self.trayIcon.activated.connect(self.toggleTrayShow)
+            self.trayIcon.show()
+
+    def trayMinimize(self):
+        """Minimize to tray based on window minimize signal.
+        """
+        if self.trayIcon and QSystemTrayIcon.isSystemTrayAvailable:
+            # skip minimize to tray if not all windows minimized
+            for control in self.localControls:
+                for window in control.windowList:
+                    if not window.isMinimized():
+                        return
+            for control in self.localControls:
+                for window in control.windowList:
+                    window.hide()
+            self.isTrayMinimized = True
+
+    def toggleTrayShow(self):
+        """Toggle show and hide application based on system tray icon click.
+        """
+        if self.isTrayMinimized:
+            for control in self.localControls:
+                for window in control.windowList:
+                    window.show()
+                    window.showNormal()
+            self.activeControl.activeWindow.treeView.setFocus()
+        else:
+            for control in self.localControls:
+                for window in control.windowList:
+                    window.hide()
+        self.isTrayMinimized = not self.isTrayMinimized
+
     def updateConfigDialog(self):
         """Update the config dialog for changes if it exists.
         """
@@ -543,6 +626,12 @@ class TreeMainControl(QObject):
         dataConfigAct.triggered.connect(self.dataConfigDialog)
         self.allActions['DataConfigType'] = dataConfigAct
 
+        dataVisualConfigAct = QAction(_('Show C&onfiguration Structure...'),
+                 self,
+                 statusTip=_('Show read-only visualization of type structure'))
+        dataVisualConfigAct.triggered.connect(self.dataVisualConfig)
+        self.allActions['DataVisualConfig'] = dataVisualConfigAct
+
         dataSortAct = QAction(_('Sor&t Nodes...'), self,
                                     statusTip=_('Define node sort operations'),
                                     checkable=True)
@@ -607,6 +696,17 @@ class TreeMainControl(QObject):
         toolsFontsAct.triggered.connect(self.toolsCustomFonts)
         self.allActions['ToolsFonts'] = toolsFontsAct
 
+        toolsColorsAct = QAction(_('Custo&mize Colors...'), self,
+                                statusTip=_('Customize GUI colors and themes'))
+        toolsColorsAct.triggered.connect(self.toolsCustomColors)
+        self.allActions['ToolsColors'] = toolsColorsAct
+
+        formatSelectAllAct =  QAction(_('&Select All'), self,
+                                   statusTip=_('Select all text in an editor'))
+        formatSelectAllAct.setEnabled(False)
+        formatSelectAllAct.triggered.connect(self.formatSelectAll)
+        self.allActions['FormatSelectAll'] = formatSelectAllAct
+
         helpBasicAct = QAction(_('&Basic Usage...'), self,
                                statusTip=_('Display basic usage instructions'))
         helpBasicAct.triggered.connect(self.helpViewBasic)
@@ -622,17 +722,6 @@ class TreeMainControl(QObject):
         helpAboutAct.triggered.connect(self.helpAbout)
         self.allActions['HelpAbout'] = helpAboutAct
 
-        formatSelectAllAct =  QAction(_('&Select All'), self,
-                                   statusTip=_('Select all text in an editor'))
-        formatSelectAllAct.setEnabled(False)
-        formatSelectAllAct.triggered.connect(self.formatSelectAll)
-        self.allActions['FormatSelectAll'] = formatSelectAllAct
-
-        helpAboutAct = QAction(_('&About TreeLine...'), self,
-                        statusTip=_('Display version info about this program'))
-        helpAboutAct.triggered.connect(self.helpAbout)
-        self.allActions['HelpAbout'] = helpAboutAct
-
         for name, action in self.allActions.items():
             icon = globalref.toolIcons.getIcon(name.lower())
             if icon:
@@ -658,6 +747,7 @@ class TreeMainControl(QObject):
                     self.activeControl.expandRootNodes()
             else:
                 self.createLocalControl()
+            self.activeControl.selectRootSpot()
 
     def fileOpen(self):
         """Prompt for a filename and open it.
@@ -725,6 +815,20 @@ class TreeMainControl(QObject):
         else:
             self.configDialog.close()
 
+    def dataVisualConfig(self):
+        """Show a TreeLine file to visualize the config structure.
+        """
+        structure = (self.activeControl.structure.treeFormats.
+                     visualConfigStructure(str(self.activeControl.
+                                               filePathObj)))
+        self.createLocalControl(treeStruct=structure, forceNewWindow=True)
+        self.activeControl.filePathObj = pathlib.Path('structure.trln')
+        self.activeControl.updateWindowCaptions()
+        self.activeControl.expandRootNodes()
+        self.activeControl.imported = True
+        win = self.activeControl.activeWindow
+        win.rightTabs.setCurrentWidget(win.outputSplitter)
+
     def dataSortDialog(self, show):
         """Show or hide the non-modal data sort nodes dialog.
 
@@ -864,6 +968,10 @@ class TreeMainControl(QObject):
             globalref.genOptions.modified):
             globalref.genOptions.writeFile()
             self.recentFiles.updateOptions()
+            if globalref.genOptions['MinToSysTray']:
+                self.createTrayIcon()
+            elif self.trayIcon:
+                self.trayIcon.hide()
             autoSaveMinutes = globalref.genOptions['AutoSaveMinutes']
             for control in self.localControls:
                 for window in control.windowList:
@@ -905,6 +1013,11 @@ class TreeMainControl(QObject):
         dialog.updateRequired.connect(self.updateCustomFonts)
         dialog.exec_()
 
+    def toolsCustomColors(self):
+        """Show dialog to customize GUI colors ans themes.
+        """
+        self.colorSet.showDialog(QApplication.activeWindow())
+
     def updateCustomFonts(self):
         """Update fonts in all windows based on a dialog signal.
         """
@@ -978,3 +1091,30 @@ class TreeMainControl(QObject):
                                          QApplication.windowIcon(),
                                          QApplication.activeWindow())
         dialog.exec_()
+
+
+def setThemeColors():
+    """Set the app colors based on options setting.
+    """
+    if globalref.genOptions['ColorTheme'] == optiondefaults.colorThemes[1]:
+        # dark theme
+        myDarkGray = QColor(53, 53, 53)
+        myVeryDarkGray = QColor(25, 25, 25)
+        myBlue = QColor(42, 130, 218)
+        palette = QPalette()
+        palette.setColor(QPalette.Window, myDarkGray)
+        palette.setColor(QPalette.WindowText, Qt.white)
+        palette.setColor(QPalette.Base, myVeryDarkGray)
+        palette.setColor(QPalette.AlternateBase, myDarkGray)
+        palette.setColor(QPalette.ToolTipBase, Qt.darkBlue)
+        palette.setColor(QPalette.ToolTipText, Qt.lightGray)
+        palette.setColor(QPalette.Text, Qt.white)
+        palette.setColor(QPalette.Button, myDarkGray)
+        palette.setColor(QPalette.ButtonText, Qt.white)
+        palette.setColor(QPalette.BrightText, Qt.red)
+        palette.setColor(QPalette.Link, myBlue)
+        palette.setColor(QPalette.Highlight, myBlue)
+        palette.setColor(QPalette.HighlightedText, Qt.black)
+        palette.setColor(QPalette.Disabled, QPalette.Text, Qt.darkGray)
+        palette.setColor(QPalette.Disabled, QPalette.ButtonText, Qt.darkGray)
+        qApp.setPalette(palette)
