@@ -9,7 +9,7 @@
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import re
@@ -331,7 +331,7 @@ class TreeView(QTreeView):
             self.selectionModel().selectSpots([])
             self.scheduleDelayedItemsLayout()
         if event.isAccepted():
-            self.model().treeModified.emit(True)
+            self.model().treeModified.emit(True, True)
 
     def toggleNoMouseSelectMode(self, active=True):
         """Set noMouseSelectMode to active or inactive.
